@@ -4,19 +4,10 @@ import zipfile
 from histdatacom.concurrency import get_pool_cpu_count
 from histdatacom.concurrency import ProcessPool
 from rich import print
+from histdatacom import config
 
 
 class _CSVs:
-    def __init__(self, args_, records_current_, records_next_):
-        # setting relationship to global outer parent
-        self.args = args_
-
-        global records_current
-        records_current = records_current_
-
-        global records_next
-        records_next = records_next_
-
     def extract_csv(self, record, args, records_current, records_next):
         try:
             if "CSV_ZIP" in record.status:
@@ -37,11 +28,11 @@ class _CSVs:
         finally:
             records_current.task_done()
 
-    def extract_csvs(self, records_current, records_next):
+    def extract_csvs(self):
 
         pool = ProcessPool(self.extract_csv,
-                           self.args,
+                           config.args,
                            "Extracting", "CSVs...",
-                           get_pool_cpu_count(self.args['cpu_utilization']))
+                           get_pool_cpu_count(config.args['cpu_utilization']))
 
-        pool(records_current, records_next)
+        pool(config.current_queue, config.next_queue)
